@@ -5,101 +5,67 @@ no strict;
 @ISA = qw( Exporter Mail::Mbox::MessageParser );
 
 use strict;
+use Carp;
+
 use Mail::Mbox::MessageParser;
+use Mail::Mbox::MessageParser::Config;
 
-use vars qw( $VERSION $DEBUG $FROM_PATTERN );
+use vars qw( $VERSION $DEBUG );
 
-$VERSION = '1.04';
+$VERSION = sprintf "%d.%02d%02d", q/1.50.0/ =~ /(\d+)/g;
 
 *DEBUG = \$Mail::Mbox::MessageParser::DEBUG;
-*FROM_PATTERN = \$Mail::Mbox::MessageParser::FROM_PATTERN;
 *dprint = \&Mail::Mbox::MessageParser::dprint;
 sub dprint;
 
-# Need this for a lookahead.
-my $READ_CHUNK_SIZE = 20000;
-
 #-------------------------------------------------------------------------------
 
 sub new
 {
-  my ($proto, $options) = @_;
+  my ($proto, $self) = @_;
 
-  my $class = ref($proto) || $proto;
-  my $self  = {};
-  bless ($self, $class);
+  carp "Need file_handle option" unless defined $self->{'file_handle'};
 
-  die "Need file_handle option" unless defined $options->{'file_handle'};
+  bless ($self, __PACKAGE__);
 
-  $self->{'file_handle'} = $options->{'file_handle'};
-
-  $self->reset();
-
-  $self->{'file_name'} = $options->{'file_name'}
-    if defined $options->{'file_name'};
-
-  $self->{'READ_CHUNK_SIZE'} = $READ_CHUNK_SIZE;
-
-  $self->_print_debug_information();
+  $self->_init();
 
   return $self;
 }
 
 #-------------------------------------------------------------------------------
 
+sub _init
+{
+  my $self = shift;
+
+  $self->{'READ_CHUNK_SIZE'} =
+    $Mail::Mbox::MessageParser::Config{'read_chunk_size'};
+
+  $self->{'CURRENT_LINE_NUMBER'} = 1;
+  $self->{'CURRENT_OFFSET'} = 0;
+
+  $self->{'READ_BUFFER'} = '';
+  $self->{'START_OF_EMAIL'} = 0;
+  $self->{'END_OF_EMAIL'} = 0;
+
+  $self->SUPER::_init();
+}
+
+#-------------------------------------------------------------------------------
+
 sub reset
 {
   my $self = shift;
 
-  if (defined $self->{'prologue'})
-  {
-    if (_IS_A_PIPE($self->{'file_handle'}))
-    {
-      dprint "Avoiding seek() on a pipe";
-    }
-    else
-    {
-      seek $self->{'file_handle'}, length($self->{'prologue'}), 0
-    }
-
-    $self->{'CURRENT_LINE_NUMBER'} = ($self->{'prologue'} =~ tr/\n//) + 1;
-    $self->{'CURRENT_OFFSET'} = length($self->{'prologue'});
-  }
-  else
-  {
-    if (_IS_A_PIPE($self->{'file_handle'}))
-    {
-      dprint "Avoiding seek() on a pipe";
-    }
-    else
-    {
-      seek $self->{'file_handle'}, 0, 0;
-    }
-
-    $self->{'CURRENT_LINE_NUMBER'} = 1;
-    $self->{'CURRENT_OFFSET'} = 0;
-  }
+  $self->{'CURRENT_LINE_NUMBER'} = ($self->{'prologue'} =~ tr/\n//) + 1;
+  $self->{'CURRENT_OFFSET'} = length($self->{'prologue'});
 
   $self->{'READ_BUFFER'} = '';
   $self->{'START_OF_EMAIL'} = 0;
   $self->{'END_OF_EMAIL'} = 0;
 
-  $self->{'end_of_file'} = 0;
-
-  $self->{'email_line_number'} = 0;
-  $self->{'email_offset'} = 0;
-  $self->{'email_length'} = 0;
-  $self->{'email_number'} = 0;
-}
-
-#-------------------------------------------------------------------------------
-
-sub _IS_A_PIPE
-{
-  my $file_handle = shift;
-
-  return (-t $file_handle || -S $file_handle || -p $file_handle ||
-    !-f $file_handle || !(seek $file_handle, 0, 1));
+  $self->SUPER::reset();
 }
 
 #-------------------------------------------------------------------------------
@@ -108,11 +74,12 @@ sub _read_prologue
 {
   my $self = shift;
 
-  dprint "Reading mailbox prologue";
+  dprint "Reading mailbox prologue using Perl";
 
   # Look for the start of the next email
   LOOK_FOR_FIRST_HEADER:
-  if ($self->{'READ_BUFFER'} =~ m/$FROM_PATTERN/mg)
+  if ($self->{'READ_BUFFER'} =~
+      m/$Mail::Mbox::MessageParser::Config{'from_pattern'}/mg)
   {
     my $start_of_email = pos($self->{'READ_BUFFER'}) - length($1);
 
@@ -187,8 +154,6 @@ sub read_next_email
 {
   my $self = shift;
 
-  dprint "Using Perl" if $DEBUG;
-
   $self->{'email_line_number'} = $self->{'CURRENT_LINE_NUMBER'};
   $self->{'email_offset'} = $self->{'CURRENT_OFFSET'};
 
@@ -196,20 +161,34 @@ sub read_next_email
 
   # Look for the start of the next email
   LOOK_FOR_NEXT_HEADER:
-  while ($self->{'READ_BUFFER'} =~ m/$FROM_PATTERN/mg)
+  while ($self->{'READ_BUFFER'} =~
+      m/$Mail::Mbox::MessageParser::Config{'from_pattern'}/mg)
   {
     $self->{'END_OF_EMAIL'} = pos($self->{'READ_BUFFER'}) - length($1);
 
     # Don't stop on email header for the first email in the buffer
     next unless $self->{'END_OF_EMAIL'};
 
+    my $endline = $self->{'endline'};
+
     # Keep looking if the header we found is part of a "Begin Included
     # Message".
-    my $end_of_string = substr($self->{'READ_BUFFER'}, $self->{'END_OF_EMAIL'}-200, 200);
-    my $endline = $self->{'endline'};
+    my $end_of_string = '';
+    my $backup_amount = 100;
+    do
+    {
+      $backup_amount *= 2;
+      $end_of_string = substr($self->{'READ_BUFFER'},
+        $self->{'END_OF_EMAIL'}-$backup_amount, $backup_amount);
+    } while (index($end_of_string, "$endline$endline") == -1 &&
+      $backup_amount < $self->{'END_OF_EMAIL'});
+
     next if $end_of_string =~
         /$endline-----(?: Begin Included Message |Original Message)-----$endline[^\r\n]*(?:$endline)*$/i;
 
+    next if $end_of_string =~
+      /$endline--[^\r\n]*${endline}Content-type:[^\r\n]*$endline(?:$endline)+$/i;
+
     # Found the next email!
     $self->{'email_length'} = $self->{'END_OF_EMAIL'}-$self->{'START_OF_EMAIL'};
     my $email = substr($self->{'READ_BUFFER'}, $self->{'START_OF_EMAIL'},
@@ -297,13 +276,13 @@ Mail::Mbox::MessageParser::Perl - A Perl-based mbox folder reader
 
   #!/usr/bin/perl
 
-  use Mail::Mbox::MessageParser::Perl;
+  use Mail::Mbox::MessageParser;
 
   my $filename = 'mail/saved-mail';
   my $filehandle = new FileHandle($filename);
 
   my $folder_reader =
-    new Mail::Mbox::MessageParser::Perl( {
+    new Mail::Mbox::MessageParser( {
       'file_name' => $filename,
       'file_handle' => $filehandle,
     } );
@@ -323,10 +302,10 @@ Mail::Mbox::MessageParser::Perl - A Perl-based mbox folder reader
 
 =head1 DESCRIPTION
 
-This module implements a perl-based mbox folder reader. Users are encouraged
-to use Mail::Mbox::MessageParser instead. The base MessageParser module will
-automatically use a faster implementation is one is available. (Although you
-can just use this module if you don't want to use caching or GNU grep.)
+This module implements a Perl-based mbox folder reader.  Users must not
+instantiate this class directly--use Mail::Mbox::MessageParser instead. The
+base MessageParser module will automatically manage the use of faster
+implementations if they can be used.
 
 =head2 METHODS AND FUNCTIONS
 
@@ -349,6 +328,8 @@ is the opened file handle to the mailbox.
 Returns a reference to a Mail::Mbox::MessageParser object, or a string
 describing the error.
 
+=back
+
 
 =head1 BUGS
 
